/**********************************************************************
 Program Name:  FilterQBOX.c
 Purpose:       Translates QBOX message format to Compuserve message
                  format for use with QL BBS Message Reader
 Author:        James Hunkins     Copyright 1994
                                  JDH Technologies
                                  Freeware
 Input:         Name of text file containing QBOX messages
                Name of output file for translating messages
 Output:        Message file matching Compuserve format for QL BBS
                  Message Reader
 EXEC Format:   EXEC FilterQBOX
                EXEC FilterQBOX;"original_mssg_file_name"
                EXEC FilterQBOX;"original_mssg_file_name, translated_file_name"
 Document:      FilterQBOX_txt
 Update Info:   06-29-94  Original
                            - pointer aware
                            - compiled with C68  V4.20
 Progress Note:
   Completed

 **********************************************************************/

#include <stdio.h>
#include <stdlib_h>
#include <string_h>
#include <ctype_h>
#include <qdos_h>

/* the following strings are what key lines start with
   the _len term is the number of characters */
#define section_line      "Message area "   /* SECTION name */
#define section_line_len  13
#define new_mssg          "#"               /* START of a new message */
#define new_mssg_len      1
#define subj_line         "Subj:"           /* SUBJECT name */
#define subj_line_len     5
#define from_line         "From:"           /* FROM name */
#define from_line_len     5
#define to_line           "To:"   /* leading spaces are not kept by C68 (?) */
#define to_line_len       3                 /* TO name */
#define special1           "End of available messages"
#define special1_len      25                /* ignore after this til next new mssg */
#define EOL               10                /* end of line character */

/* C68 declarations */
char _prog_name[] = "QBOX Filter";
char _copyright[] = "(c)1994 JD Hunkins";
char _version[] = "JDH Technologies  V0.00";
/* void consetup_title(chanid_t); */
void (*_consetup)() = consetup_title;
/* char *_endmsg = NULL; */

/* Global Variables */
FILE *in;   /* input file to be translated */
FILE *out;  /* translated file */
long QL_stdout  =  0;     /* QL channel for standard output */
long line_ctr = 0;        /* current line read in, used for error reporting */


/* prototype statements */
int  translate();
void error_out(int finish, int err_no, char other[]);
void key_pause(int sound1);


/***********************************************************************
 Function:  main
 Input:     argv[0] = program name
            argv[1] = input message file name (optional)
            argv[2] = output message file name (optional)
            argc = number of arguments given in EXEC line
 ***********************************************************************/

int main(int argc, char *argv[])
{
char line[255];       /* stores input strings */
char *pline;          /* pointer to line, used for changing starting position */
char file_name[101];
int  success;

  /* initialize and open files */
  QL_stdout = fgetchid(stdout);   /* get standard output channel no */

  if (argc > 3)   /* too many arguments */
    error_out(1,1,"");

  /* get input file name */
  if (argc == 1)
    {
    printf("Enter file containing messages: ");
    gets(line);
    sscanf(line, "%s", &file_name);
    printf("\n");
    }
  else
    strcpy(file_name, argv[1]);

  if ((in = fopen(file_name, "rt")) == NULL)
    error_out(1,2,file_name);

  /* get output file name */
  if (argc < 3)
    {
    printf("Enter destination file name [%s_mrd] : ", file_name);
    gets(line);

    if (line[0] == 0)
      strcat(file_name, "_mrd");   /* !!!!! */
    else
      sscanf(line, "%s", &file_name);
    printf("\n");
    }
  else
    strcpy(file_name, argv[2]);
  if ((out = fopen(file_name, "wt")) == NULL)
    error_out(1,3,file_name);

  /* translate file */
  if (translate() == 0)    /* calls translate function */
    {
    printf("No messages found");
    io_delete("file_name");   /* delete output file */
    }
  fclose(in);
  error_out(1,0,"\n\n> Program Closed <\n ");
}


/*********************************************************************
 Function:    translate
 Purpose:     reads, translates from QBOX format to native format (CIS)
                for QL BBS Message reader
 Return:      number of messages translated
**********************************************************************/

int translate()
{
char line[255];       /* stores input strings */
char *pline;          /* pointer to line, used for changing starting position */
char string1[41], string2[41], string3[41];
int  mssg_count = 0;  /* counts how messages found */
char section_name[41];    /* holds SECTION name for output file */
char section_number[41];  /* holds SECTION number for output file */
char subj[41];            /* holds SUBJECT name for output file */
char to[41];              /* holds TO name for output file */
char from[41];            /* holds FROM name for output file */
char date[41];            /* holds DATE for output file */
char mssg_no[41];         /* holds BBS format MESSAGE NUMBER for output file */
int  done = 0;            /* 0 = indicates not done */
int  mssg_done = 0;       /* 0 = indicates not done */
int i;
int  mssg_output = 0;     /* 0 = no messages output to output file */

  /* read in messages and translate header/end format to native format (CIS) */

  printf("TRANSLATING MSSG #:    0\b\b\b\b");   /* message # and back spaces */

  strcpy(section_name, "Unknown");   /* default values in case section_line */
  strcpy(section_number, "??");      /*   not found before first message */

  while((done == 0) && (!feof(in)))  /* loop until message found or end of file */
    {
    /* find start of message */
    while(!feof(in))
      {
      fgets(line, 255, in);          /* read in next line */
      line_ctr = line_ctr + 1;       /* increment line counter */

      /* check to see if line starts with NEW_MSSG key word(s) */
      if (strncmp(new_mssg, line, new_mssg_len) == 0)
        {
        /* message found, break from loop */
        mssg_output = 1;
        break;
        }

      /* check to see if line starts with SPECIAL1 or SECTION key word(s),
         if so, don't output lines read until another message found
      */
      if (strncmp(special1, line, special1_len) == 0)
        mssg_output = 0;  /* says to not output until new message starts */
      else if (strncmp(section_line, line, section_line_len) == 0)
        {
        /* found new section name, so get section (Area) name and number */
        pline = line + 13;
        sscanf(pline, "%40s", section_number);
        section_number[strlen(section_number)-1] = 0;   /* remove ':' at end */
        pline = strchr(line, ':') + 2;
        sscanf(pline, "%40s", section_name);
        mssg_output = 0;  /* says to not output until new message starts */
        }
      else if (mssg_output == 1)
        fprintf(out, "%s", line);   /* output line read in to output file */
      }  /* start of message found */

    if (feof(in))
      {
      done = 1;
      break;
      }
    mssg_count = mssg_count + 1;    /* message found, so increment count */
    printf("%4i\b\b\b\b", mssg_count);

    /* get header information - start with mssg # and date */
    sscanf(line, "%40s %40s %10s %10s %10s", mssg_no, string1, date, string1, string2);
    strcat(date, "-");        /* add separators and merge date strings */
    strcat(date, string1);
    strcat(date, "-");
    strcat(date, string2);
    date[strlen(date)-1] = 0;   /* remove comma at end */

    /* now look for FROM line */
    fgets(line, 255, in);
    line_ctr = line_ctr + 1;
    sscanf(line, "%40s", from);
    if (strncmp(from_line, from, from_line_len) !=0)  /* check for FROM: */
      {
      error_out(0, -1, mssg_no);
      continue;
      }
    strncpy(from, line+6, 40);

    /* next line should be TO line */
    fgets(line, 255, in);
    line_ctr = line_ctr + 1;
    sscanf(line, "%40s", to);
    if (strncmp(to_line, to, to_line_len) != 0)  /* check for TO: */
      {
      error_out(0, -2, mssg_no);
      continue;
      }
    strncpy(to, line+6, 40);

    /* now for the SUBJECT line */
    fgets(line, 255, in);
    line_ctr = line_ctr + 1;
    sscanf(line, "%40s", subj);
    if (strncmp(subj_line, subj, subj_line_len) != 0)  /* check for subject */
      {
      error_out(0, -3, mssg_no);
      continue;
      }
    strncpy(subj, line+6, 40);

    /* strip off any misc EOL, in case QBOX lines < 40 */
    if (from[strlen(from)-1] == EOL)
      from[strlen(from)-1] = 0;
    if (to[strlen(to)-1] == EOL)
      to[strlen(to)-1] = 0;
    if (subj[strlen(subj)-1] == EOL)
      subj[strlen(subj)-1] = 0;

    /* output to header to output file in native format */
    fprintf(out, "#: %s %s/%s\n", mssg_no+1, section_number, section_name);
    fprintf(out, "    %s\n", date);
    fprintf(out, "Sb: %s\n", subj);
    fprintf(out, "Fm: %s\n", from);
    fprintf(out, "To: %s\n", to);

    /* now just read in rest of message lines and send to output file
       - continue until new message of end of messages line or EOF
    */
    }

  return mssg_count;
}

/*********************************************************************
 Function:    error_out
 Purpose:     outputs error messages
 Input:       finish < 0, return as normal, -(#) = current mssg
                     = 1, exit(1) from program
              err_no = 0, quiet exit from program
                     > 0, ERROR message
                     < 0, WARNING message

 *********************************************************************/

void error_out(int finish, int err_no, char other[])
{
int i;

  switch(err_no)
    {
    case 0 :
      printf("%s\n", other);
      break;
    case 1 :
      printf("\nERROR: only 2 arguments allowed in command line...\n");
      break;
    case 2 :
      printf("\nERROR: could not open input file [%s]...\n", other);
      break;
    case 3 :
      printf("\nERROR: could not open output file [%s]...\n", other);
      break;
    case -1 :
      printf("\nWARNING: FROM line in header is wrong/missing...\n");
      printf("   Skipping message [%s]  Line #%i\n", other, line_ctr);
      key_pause(0);
      break;
    case -2 :
      printf("\nWARNING: TO line in header is wrong/missing\n");
      printf("   Skipping message [%s]  Line #%i\n", other, line_ctr);
      key_pause(0);
      break;
    case -3 :
      printf("\nWARNING: SUBJECT line in header is wrong/missing...\n");
      printf("   Skipping message [%s]  Line #%i\n", other, line_ctr);
      key_pause(0);
      break;
    default :
      printf("\nERROR/WARNING: unknown error [line #%i]...\n", line_ctr);
      key_pause(0);
      break;
    }

  if (finish > 0)
    {
    if (err_no != 0)
      printf("  >> Program Terminated << \n\n");
    for (i=1; i<1000; i++);
    exit(1);
    }

}


/***********************************************
 key_press  : waits for key input
   - input  : sound1 = 1 -> beep, 0 -> no beep
   - output : none

************************************************/

void key_pause(int sound1)
{
  if (sound1 == 1)
    beep(2400,10000);
  printf("\nPress any key to continue.. ");
  getch();
  printf("\n\n");
}
